package TradeGui;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JLabel;

import misc.Message;

import org.jdom.input.*;
import org.jdom.*;
import org.jdom.output.XMLOutputter;
import org.jdom.output.Format;

import balancingGui.LoadingPanel;



import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.Dimension;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

public class CreateTraderoutesPanel extends JPanel{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -6821139887068936454L;

	private JButton btSave;
	
	private JComboBox planetDropDown;
	private JComboBox planetDropDown2;
	
	private JLabel lbOrigin;
	private JLabel lbGoal;
	private JLabel lbHyperspace;
	private JLabel lbCredits;
	
	private JTextField tfHyperspace;
	private JTextField tfCredits;
	
	protected JPanel panelCenter;
	protected JPanel panelSouth;
	protected JPanel panelNorth;
	
	private File planetsXml = new File("PLANETS.XML");
	private File traderoutesXml = new File("TRADEROUTES.XML");
	
	private Document universe = null;
	private Document allRoutes = null;
	
	private GalaxyMapPanel gm;
	
	private String[] planetCoords;
	private Point[] trueCoords;
	private String[] planetNames;
	private Point[] currentPlanets = new Point[2];
	private String [] tradeNames;
	private String[] orgCoords;
	
	public CreateTraderoutesPanel (){
		
		lbOrigin = new JLabel(LoadingPanel.STARTPLANET);
		lbGoal = new JLabel (LoadingPanel.DESTPLANET);
		panelCenter = new JPanel();
		panelSouth = new JPanel();
		panelNorth = new JPanel();
		planetDropDown = new JComboBox(this.getPlanetNames());
		planetDropDown.addItemListener(new DropDownSelectActionListener());
		planetDropDown2 = new JComboBox(this.getPlanetNames());
		planetDropDown2.addItemListener(new DropDownSelectActionListener());
		btSave = new JButton (LoadingPanel.BTCREATE);
		btSave.addActionListener(new BtSaveActionListener());
		
		lbHyperspace = new JLabel(LoadingPanel.HYPERSPACESPEED);
		lbCredits = new JLabel(LoadingPanel.CREDITFACTOR);
		tfHyperspace = new JTextField(5);
		tfCredits = new JTextField(5);

		gm = new GalaxyMapPanel();
		gm.setCoords(trueCoords, planetNames,orgCoords);
		gm.setTraderoutes(this.getTraderoute());
		
		panelNorth.add(lbOrigin,BorderLayout.NORTH);
		panelNorth.add(planetDropDown);
		panelNorth.add(lbGoal,BorderLayout.NORTH);
		panelNorth.add(planetDropDown2);
		panelCenter.add(lbHyperspace);
		panelCenter.add(tfHyperspace);
		panelCenter.add(lbCredits);
		panelCenter.add(tfCredits);
		panelSouth.add(btSave);
		
		
		
		this.add(panelNorth, BorderLayout.NORTH);
		this.add(panelCenter, BorderLayout.CENTER);
		this.add(panelSouth, BorderLayout.SOUTH);
		this.add(gm, BorderLayout.EAST);
		
		this.setPreferredSize(new Dimension(1100,950));
		this.repaint();
		
		
	}
	
	protected void paintComponent( Graphics g ){
		gm.paintComponent(g);
		
	}
	
	public String[] getPlanetNames(){
		SAXBuilder builder = new SAXBuilder();
		
		try {
			universe = builder.build(planetsXml);
		} catch (JDOMException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			if(e.getClass()==(com.sun.org.apache.xerces.internal.impl.io.MalformedByteSequenceException.class))
				Message.showErrorMessage(e, LoadingPanel.READERROR,LoadingPanel.INVALIDFILE + '\n' +LoadingPanel.EDITORSHUTDOWN,true);
			else{
				Message.showErrorMessage(e, LoadingPanel.FILENOTFOUND,LoadingPanel.EDITORSHUTDOWN,true);				
			}
		}
		Element planets = universe.getRootElement();
		List planetList = planets.getChildren( "Planet" );
		
		planetNames = new String [planetList.size()];
		planetCoords = new String [planetList.size()];
		trueCoords = new Point [planetList.size()];
		orgCoords = new String [planetList.size()];
		for (int i=1;i<planetList.size();i++){
			Element temp = (Element)planetList.get(i);
			planetNames[i] = temp.getAttribute( "Name" ).getValue();
			planetCoords[i] = temp.getChild("Galactic_Position").getValue();
		}
		refineCoords();
		return planetNames;
	}
	
	
	
	public void refineCoords(){
		
		for (int i=1;i<planetCoords.length;i++){
			String[] temp = new String[3];
			temp=(String[]) planetCoords[i].split(",").clone();
			orgCoords[i]=new String((int)Double.parseDouble(temp[0])+","+((int)Double.parseDouble(temp[1])));
			trueCoords[i]=new Point((int)Double.parseDouble(temp[0])+LoadingPanel.gX,Math.abs((int)Double.parseDouble(temp[1])-LoadingPanel.gY));
		}
	}
	
	public Element openTrade(){
		SAXBuilder builder = new SAXBuilder();
		try {
			allRoutes = builder.build(traderoutesXml);
		} catch (JDOMException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			if(e.getClass()==(com.sun.org.apache.xerces.internal.impl.io.MalformedByteSequenceException.class))
				Message.showErrorMessage(e, LoadingPanel.READERROR,LoadingPanel.INVALIDFILE + '\n' +LoadingPanel.EDITORSHUTDOWN,true);
			else{
				Message.showErrorMessage(e, LoadingPanel.FILENOTFOUND,LoadingPanel.EDITORSHUTDOWN,true);				
			}
		}
		return allRoutes.getRootElement();
	}
	
	public String[] getAllTradeRoutes(){
		SAXBuilder builder = new SAXBuilder();
		try {
			allRoutes = builder.build(traderoutesXml);
		} catch (JDOMException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			if(e.getClass()==(com.sun.org.apache.xerces.internal.impl.io.MalformedByteSequenceException.class))
				Message.showErrorMessage(e, LoadingPanel.READERROR,LoadingPanel.INVALIDFILE + '\n' +LoadingPanel.EDITORSHUTDOWN,true);
			else{
				Message.showErrorMessage(e, LoadingPanel.FILENOTFOUND,LoadingPanel.EDITORSHUTDOWN,true);				
			}
		}
		Element traderoutes = allRoutes.getRootElement();
		List tradeList = traderoutes.getChildren( "TradeRoute" );
		
		tradeNames = new String [tradeList.size()];
		for (int i=0;i<tradeList.size();i++){
			Element temp = (Element)tradeList.get(i);
			tradeNames[i] = temp.getAttribute( "Name" ).getValue();
		}
		return tradeNames;
	}

	
	public Point[] getTraderoute(){
		String[] allRoutes = (String[]) this.getAllTradeRoutes().clone(); 
		String org=null,dest=null;
		Point[] coords1= new Point[allRoutes.length];
		Point[] coords2= new Point[allRoutes.length];
		Point[] coords = new Point[allRoutes.length*2];
		int c1=0,c2=0;
		Element root = openTrade();
		List all = root.getChildren();
		for (int j = 0; j < allRoutes.length; j++) {
			for (Object o : all) { // get the Origin and Destination Planets of
									// the Traderoute
				Element temp = (Element) o;
				//System.out.println(allRoutes[j]);
				String s = allRoutes[j];
				//System.out.println(s);
				if ((temp.getAttributeValue("Name")).equals(s)) {
					//System.out.println(temp.getChild("Point_A").getValue()+"_"+temp.getChild("Point_B").getValue());
					org = temp.getChild("Point_A").getValue();
					dest = temp.getChild("Point_B").getValue();
					break;
				}
				
			}
			for (int i = 1; i < planetNames.length; i++) { // get the corresponding Coordinates of the Planets
				if (planetNames[i].equals(org))
					c1 = i;
				if (planetNames[i].equals(dest))
					c2 = i;
				if (c1 != 0 && c2 != 0)
					break;
				
			}
			coords1[j] = trueCoords[c1];
			coords2[j] = trueCoords[c2];
			c1=0;
			c2=0;
			coords[j]=coords1[j];
			coords[j+coords1.length]=coords2[j];
			//System.out.println("Coords von j: " + coords[j]+","+ "Coords von j+1: "+ coords[coords1.length+j]);
		}
		return coords;
	}
	
	protected class BtSaveActionListener implements ActionListener {
		public Element addElement(String name, String content){
			Element temp = new Element( name );
			temp.addContent( content );
			return temp;
		}
		public void actionPerformed(ActionEvent e)  {
			Element rootElem;
			String[] data = new String[12];
			Element[] tradeRoute = new Element[6];
			data[0]=(String)planetDropDown.getSelectedItem();
			data[1]= (String)planetDropDown2.getSelectedItem();
			data[2]=tfHyperspace.getText();
			data[3]="0.0";
			data[4]=tfCredits.getText();
			data[5]="DEFAULT";
			data[6]="Point_A";
			data[7]="Point_B";
			data[8]="HS_Speed_Factor";
			data[9]="Political_Control_Gain";
			data[10]="Credit_Gain_Factor";
			data[11]="Visible_Line_Name";
			for (int i=0;i<6;i++ ){
				tradeRoute[i]=this.addElement(data[i+6],data[i]);
			}
			rootElem = openTrade();
			Element newTrade = new Element ("TradeRoute");
			for (Element temp : tradeRoute){
				newTrade.addContent(temp);
			}
			newTrade.setAttribute( "Name", data[0]+"_"+data[1]);
			rootElem.addContent(newTrade);
			

			System.out.println(newTrade.getAttribute( "Name" ).getValue());
//			for (Object temp : newTrade.getChildren()){
//				Element x = (Element) temp;
//				System.out.println(x.getText());
//			}
			
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat() );
			try {
				out.output(allRoutes, new FileWriter(traderoutesXml));
			} catch (IOException f) {
				// TODO Auto-generated catch block
				f.printStackTrace();
			}
			gm.setTraderoutes(getTraderoute());
			repaint();
			tradeWriter();
		}	
	}
	
	protected class DropDownSelectActionListener implements ItemListener {
		public void itemStateChanged(ItemEvent ie) {
			String selected1 = ((String)planetDropDown.getSelectedItem());
			if(selected1 == null) {
				return;
			}
			String selected2 = ((String)planetDropDown2.getSelectedItem());
			if(selected2 == null) {
				return;
			}
			int c1=0, c2=0;
			for (int i = 1; i < planetNames.length; i++) { // get the
				// corresponding
				// Coordinates of
				// the Planets
				if (planetNames[i].equals(selected1))
					c1 = i;
				if (planetNames[i].equals(selected2))
					c2 = i;
				if (c1 != 0 && c2 != 0)
					break;

			}
			currentPlanets[0]=trueCoords[c1];
			currentPlanets[1]=trueCoords[c2];
			gm.drawCurrentPlanets(currentPlanets);
			gm.repaint();
			
		}
	}
	
	private void tradeWriter(){
		FileOutputStream fos;
		try {
			String com = ",";
			fos = new FileOutputStream( "traderoutes.txt" );
			for(String s : tradeNames){
				fos.write( s.getBytes() );
				fos.write(com.getBytes());
			}
		    fos.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	

	
	
	
}
